from colorama import Fore
import sys
ERROR=0
WARNING=1
INFO=2
def debug_output(text,level):
    val=['[ERROR]','[WARNING]','[INFO]']
    color=[Fore.RED,Fore.YELLOW,Fore.LIGHTBLUE_EX]
    print(color[level]+val[level]+Fore.RESET,' '+text,file=sys.stderr,flush=True)



from PIL import Image,ImageFont
import numpy as np
import cv2
from .imageproc import *
from pygments.lexers import *
from pygments import *
from .config import *
from .easy import *
class Background():
    """
    Class for background of code screenshot.
    """
    def __init__(self,obj):
        if isinstance(obj,tuple): # Background is a color.
            if isinstance(obj[0],tuple):
                self.type=0
                self.lineno_bg=obj[0]
                self.code_bg=obj[1]
            else:
                self.lineno_bg=obj
                self.code_bg=obj
                self.type=0
        else: # Background is an image.
            self.type=1
            self.bg=obj
    def color(self,code,lineno):
        if self.type:
            whole=image_merge_row(lineno,code)
            bg=cv2.resize(self.bg,(whole.shape[1],whole.shape[0]),interpolation=cv2.INTER_LINEAR)
            for i in range(whole.shape[0]):
                for j in range(whole.shape[1]):
                    if whole[i][j][3]==0:
                        whole[i][j][3]=255
                        whole[i][j][:3]=bg[i][j][:3]
            return np.array(Image.fromarray(whole).convert('RGB'))
        else:
            for i in range(code.shape[0]):
                for j in range(code.shape[1]):
                    if code[i][j][3]==0:
                        code[i][j][3]=255
                        code[i][j][:3]=np.array(self.code_bg,dtype=np.uint8)
            for i in range(lineno.shape[0]):
                for j in range(lineno.shape[1]):
                    if lineno[i][j][3]==0:
                        lineno[i][j][3]=255
                        lineno[i][j][:3]=np.array(self.lineno_bg,dtype=np.uint8)
            return np.array(Image.fromarray(image_merge_row(lineno,code)).convert('RGB'))
def fromstr(string):
    def hex2rgb(x):
        return (int(x[:2],16),int(x[2:4],16),int(x[4:6],16))
    if '.' in string:
        return Background(image_open(string))
    elif len(string)==6:
        return Background(hex2rgb(string))
    else:
        return Background((hex2rgb(string[:6]),hex2rgb(string[6:])))



"""
Tools for token type and highlight config lookup.
"""
from pygments.token import *
from json import loads
class TokenChecker(object):
    """
    Token alias lookup.
    """
    def __init__(self):
        """
        All 30 token aliases.
        """
        self.config=[
            ('keyword',Keyword),
            ('builtin', Name.Builtin),
            ('decorator', Name.Decorator),
            ('class', Name.Class),
            ('func', Name.Function),
            ('exception',Name.Exception),
            ('identifier',Name),
            ('docstring',String.Doc),
            ('char',String.Char),
            ('escape',String.Escape),
            ('affix',String.Affix),
            ('regex',String.Regex),
            ('string',String),
            ('bin',Number.Bin),
            ('oct',Number.Oct),
            ('hex',Number.Hex),
            ('float',Number.Float),
            ('int',Number.Integer),
            ('date',Literal.Date),
            ('operator',Operator),
            ('punc',Punctuation),
            ('shebang',Comment.Hashbang),
            ('multiline-comment',Comment.Mutiline),
            ('preprocessor',Comment.Preproc),
            ('preprocessor-file',Comment.PreprocFile),
            ('comment',Comment.Single),
            ('whitespace',Whitespace),
            ('unknown',Text),
            ('error',Error),
            ('other',Other)
        ]
    def __call__(self,tok):
        for i,j in self.config:
            if is_token_subtype(tok,j):
                return i
    def __repr__(self):
        return '<TokenChecker>'
    def __str__(self):
        return self.__repr__()
    def get_all_names(self):
        return [i[0] for i in self.config]
class ColorConfig(object):
    """
    Syntax highlight config.
    """
    def __init__(self,cfdct,default=(0,0,0)):
        self._cf=cfdct
        self._d=default
    def __call__(self,tokname):
        if tokname in self._cf:
            return self._cf[tokname]
        return self._d
    def __repr__(self):
        return '<ColorConfig {}, default is {}>'.format(self._cf,self._d)
    def __str__(self):
        return self.__repr__()
def color_config_from_json(json):
    """
    Get syntax highlight config from a JSON string
    for example:
    {
        "config":{
            "keyword":[127,0,0],
            "builtin":[0,0,127]
            }
    }
    :param json: A valid JSON string.
    :return: Syntax highlight config from the JSON string
    """
    dc=loads(json)
    if 'config' not in dc:
        raise ValueError("A color config JSON file should have a 'config' key")
    else:
        if 'default' in dc:
            return ColorConfig(dc['config'],default=dc['default'])
        else:
            return ColorConfig(dc['config'])